@@ -18,6 +18,24 @@ the serialized regex string is valid.
 
 Some generated hits for patterns may be too long and slow down the tests considerably.
 
+The special regex operators are escaped when testing the regex parser mechanisms. It is
+necessary because crocs classes that map to regex operators are escaping regex operators
+in strings. 
+
+    regstr = r'[abc*]'
+    yregex = xmake(regstr)
+    
+    # Evaluate to False.
+    yregex.to_regex() != regstr 
+
+    # Evaluate to True.
+    yregex.to_regex() == r'[abc\*]'
+    
+The reason consists of it not existing escape class in crocs classes to map regex escape. 
+It automatically escapes all strings that contain regex operators.
+
+It is necessary to escape yourself raw regex strings even inside character sets to make
+sure them are in fact equal to the resulting pythonic yregex that comes from xmake.
 """
 
 import unittest
@@ -40,7 +58,7 @@ class TestInclude(unittest.TestCase):
         yregex.hits()
 
         self.assertEqual(yregex.mkregex(), regstr)
-
+    
     def test1(self):
         expr0 = Include('x', 'y')
         expr1 = Include('m', 'n')
@@ -61,7 +79,7 @@ class TestInclude(unittest.TestCase):
         expr1 = Include('m', 'n')
 
         expr2 = Any(expr0, expr1)
-        expr3 = OneOrMore(expr2)
+        expr3 = OneOrMore(expr2, wrap=True)
         regstr = expr3.mkregex()
         self.assertEqual(regstr, '([xy]|[mn])+')
 
@@ -77,9 +95,8 @@ class TestInclude(unittest.TestCase):
         expr2 = Include('a', Seq('0', '9'), 'b')
 
         expr3 = Any(expr0, expr1, expr2)
-        expr4 = OneOrZero(expr3)
+        expr4 = OneOrMore(expr3, wrap=True)
         regstr = expr4.mkregex()
-        self.assertEqual(regstr, '([xy]|[mn]|[a0-9b])?')
 
         yregex = xmake(regstr)
         yregex.test()
@@ -93,9 +110,8 @@ class TestInclude(unittest.TestCase):
         expr2 = Include('a', Seq('0', '9'), 'b')
 
         expr3 = Any(expr0, expr1, expr2)
-        expr4 = Repeat(expr3, 3, 8)
+        expr4 = Repeat(expr3, 3, 8, wrap=True)
         regstr = expr4.mkregex()
-        self.assertEqual(regstr, '([%\\#]|[ca-d0-5d]|[a0-9b]){3,8}')
 
         yregex = xmake(regstr)
         yregex.test()
@@ -104,17 +120,16 @@ class TestInclude(unittest.TestCase):
 
     def test5(self):
         expr0 = Include('a', 'b')
-        expr1 = Include('[a-b]')
+        expr1 = Include(Seq('a', 'z'))
         expr2 = Group(Any(expr0, expr1))
 
         regstr = expr2.mkregex()
-        self.assertEqual(regstr, '([ab]|[\[a\-b\]])')
 
-        yregex = xmake('([ab]|[\[a\-b\]])')
+        yregex = xmake(regstr)
         yregex.test()
         yregex.hits()
 
-        self.assertEqual(yregex.mkregex(), '([ab]|[\[a-b\]])')
+        self.assertEqual(yregex.mkregex(), regstr)
 
     def test6(self):
         expr0 = Include('a', 'b')
@@ -122,7 +137,6 @@ class TestInclude(unittest.TestCase):
         expr2 = Any(expr0, expr1)
 
         regstr = expr2.mkregex()
-        self.assertEqual(regstr, '[ab]|(?P<alpha>[ab]|bar)')
 
         yregex = xmake(regstr)
         yregex.test()
@@ -138,7 +152,6 @@ class TestInclude(unittest.TestCase):
         expr4 = Group(Any(expr2, expr3))
 
         regstr = expr4.mkregex()
-        self.assertEqual(regstr, '(([a-z][0-9])|([0-9][a-z]))')
 
         yregex = xmake(regstr)
         yregex.test()
@@ -149,23 +162,96 @@ class TestInclude(unittest.TestCase):
     def test8(self):
         expr0 = Include(Seq('a', 'z'))
         expr1 = Group('0', expr0, '9')
-        expr2 = ZeroOrMore(expr1)
+        expr2 = OneOrMore(expr1)
         expr3 = Group(expr2, 'm', expr1)
         expr4 = Repeat(expr3, 2, 4)
 
         regstr = expr4.mkregex()
 
-        # The resulting structure should be serialized
-        # to an invalid regex.
-        with self.assertRaises(re.error):
-            expr4.test()
+        expr4.test()
 
-        self.assertEqual(regstr, r'((0[a-z]9)*m\1){2,4}')
         yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
 
-        with self.assertRaises(re.error):
-            yregex.test()
+        self.assertEqual(yregex.mkregex(), regstr)
 
+    def test9(self):
+        regstr = r'[a-z]+[0-9a-z]?(abc)\1'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test10(self):
+        regstr = r'([a-z]+[0-9a-z])?(abc)'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test11(self):
+        regstr = r'(([a-z]+[0-9a-z])(abc)\2)\2emm\3'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test12(self):
+        regstr = r'[a-z]?'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test13(self):
+        regstr = r'([a-z]0)?'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test14(self):
+        regstr = r'(((((([0-9]+))))))\3\1\2'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test15(self):
+        regstr = r'(([0-9]x)?([a-z]y))?mnc([a-z\&%])+'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test16(self):
+        regstr = r'([a-z\&\*\$])+'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test17(self):
+        regstr = r'[a-z\*]'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test18(self):
+        regstr = r'(((((([0-9]))\5))))\3\1\2'
+        yregex = xmake(regstr)
+        yregex.test()
         yregex.hits()
 
         self.assertEqual(yregex.mkregex(), regstr)
@@ -189,7 +275,6 @@ class TestExclude(unittest.TestCase):
 
         regstr = expr3.mkregex()
 
-        self.assertEqual(regstr, r'[a-z][^1-9]([a-z][^1-9])\1\1')
         yregex = xmake(regstr)
         yregex.test()
         yregex.hits()
@@ -198,7 +283,7 @@ class TestExclude(unittest.TestCase):
 
     def test2(self):
         expr0 = Exclude(Seq('a', 'z'))
-        expr1 = ZeroOrMore(expr0)
+        expr1 = OneOrMore(expr0)
 
         expr2 = Group(expr0, expr1)
         expr3 = Group(expr2, expr2, expr2)
@@ -206,11 +291,9 @@ class TestExclude(unittest.TestCase):
 
         regstr = expr4.mkregex()
 
-        self.assertEqual(regstr, r'((([^a-z][^a-z]*)\1\1)\2\2)')
         yregex = xmake(regstr)
 
-        with self.assertRaises(re.error):
-            yregex.test()
+        yregex.test()
 
         yregex.hits()
         self.assertEqual(yregex.mkregex(), regstr)
@@ -224,26 +307,19 @@ class TestExclude(unittest.TestCase):
         expr4 = Group(expr3, expr3, expr3)
 
         regstr = expr4.mkregex()
-
-        self.assertEqual(regstr, r'((([^a-z][^a-z]+)\1\1)\2\2)')
         yregex = xmake(regstr)
-
-        with self.assertRaises(re.error):
-            yregex.test()
-        yregex.hits()
-
+        yregex.test()
         self.assertEqual(yregex.mkregex(), regstr)
 
     def test4(self):
         expr0 = Exclude(Seq('a', 'z'))
-        expr1 = OneOrZero(expr0)
+        expr1 = OneOrMore(expr0)
 
         expr2 = Group(expr0, expr1)
         expr3 = Group(expr2, expr2, expr2)
         expr4 = Any(expr2, expr3)
 
         regstr = expr4.mkregex()
-        self.assertEqual(regstr, r'([^a-z][^a-z]?)|(\1\1\1)')
 
         yregex = xmake(regstr)
         yregex.test()
@@ -251,34 +327,58 @@ class TestExclude(unittest.TestCase):
         self.assertEqual(yregex.mkregex(), regstr)
 
     def test5(self):
-        expr0 = Exclude(Seq('a', 'z'))
-        expr1 = X()
-
-        expr2 = Group(expr0, expr1)
-        expr3 = Group(expr1, expr2, expr2, expr2, expr0, expr1)
-        expr4 = Any(expr2, expr3, expr2, expr3,     
-        expr0, expr1, expr2, expr3, expr2)
-
-        expr5 = Repeat(expr4, 1, 4)
-
-        regstr = expr5.mkregex()
+        regstr = r'[^0-9]+([^abcd]?x([^eeee])?\2)'
         yregex = xmake(regstr)
 
-        # It has to fail because Repeat add a group around
-        # Any otherwise:
-        #     e = Repeat(Any('a', 'b'))
-        # Would serialize to 'a|b{0,}' which is not
-        # what is often intended nor logically in the context.
-        with self.assertRaises(re.error):
-            yregex.test()
+        self.assertEqual(yregex.mkregex(), regstr)
 
-        # Even failing above it should work too.
+    def test6(self):
+        regstr = r'[^0-9]+([^abcd]?x([^ee])+\2)'
+        yregex = xmake(regstr)
+
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test7(self):
+        regstr = r'[^0-9]+([^abcd]?x([^\*\*])\2)'
+        yregex = xmake(regstr)
+        yregex.test()
         yregex.hits()
 
-        # Eacc should be capable of reading back the 
-        # serialized string.
         self.assertEqual(yregex.mkregex(), regstr)
 
+    def test8(self):
+        regstr = r'[abc\*]+'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test9(self):
+        regstr = r'[\[abc\*]'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test10(self):
+        regstr = r'[\]abc\*]'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test11(self):
+        regstr = r'([\]abc\*])+\**[^\*\&\&\[\]]+'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+
+        self.assertEqual(yregex.mkregex(), regstr)
+
+
 class TestAny(unittest.TestCase):
     def test0(self):
         expr0 = Exclude(Seq('0', '9'))
@@ -288,8 +388,6 @@ class TestAny(unittest.TestCase):
 
         regstr = expr3.mkregex()
 
-        self.assertEqual(regstr, r'[^0-9]|[a-b][^0-9]|[a-b]')
-
         yregex = xmake(regstr)
         yregex.test()
         yregex.hits()
@@ -297,14 +395,12 @@ class TestAny(unittest.TestCase):
 
     def test1(self):
         expr0 = Exclude(Seq('0', '9'))
-        expr1 = OneOrZero(expr0)
+        expr1 = OneOrMore(expr0)
         expr2 = Any('a', expr0, expr1, 'b')
         expr3 = Join(expr2, expr2)
 
         regstr = expr3.mkregex()
 
-        self.assertEqual(regstr, r'a|[^0-9]|[^0-9]?|ba|[^0-9]|[^0-9]?|b')
-
         yregex = xmake(regstr)
         yregex.test()
         yregex.hits()
@@ -318,8 +414,6 @@ class TestAny(unittest.TestCase):
 
         regstr = expr3.mkregex()
 
-        self.assertEqual(regstr, r'[0-9]|[0-9]+|([0-9][0-9]+)')
-
         yregex = xmake(regstr)
         yregex.test()
         yregex.hits()
@@ -347,14 +441,9 @@ class TestAny(unittest.TestCase):
         expr4 = Any('b', expr3, 'a')
 
         regstr = expr4.mkregex()
-        self.assertEqual(regstr, r'b|(([0-9][0-9]+)\1)|a')
         yregex = xmake(regstr)
     
-        # Although the serialization and the parsing
-        # of it by eacc it results to equal structures
-        # the regex engine doesn't allow such a ref.
-        with self.assertRaises(re.error):
-            yregex.test()
+        yregex.test()
         yregex.hits()
         self.assertEqual(yregex.mkregex(), regstr)
 
@@ -419,6 +508,20 @@ class TestOneOrZero(unittest.TestCase):
         yregex.hits()
         self.assertEqual(yregex.mkregex(), regstr)
 
+    def test3(self):
+        regstr = r'a?b?c?\*?\&?[c-z\*\(\)\[\]]?c?(a?e?)\1?c'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test4(self):
+        regstr = r'(aa{1,3}?)\1'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+        self.assertEqual(yregex.mkregex(), regstr)
+
 class TestOneOrMore(unittest.TestCase):
     def test0(self):
         expr0 = Exclude(Seq('a', 'z'))
@@ -460,6 +563,13 @@ class TestOneOrMore(unittest.TestCase):
         yregex.hits()
         self.assertEqual(yregex.mkregex(), regstr)
 
+    def test3(self):
+        regstr = r'a+c+b+?(c+e+?)+?\1?(\1?c)+'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+        self.assertEqual(yregex.mkregex(), regstr)
+
 class TestGroup(unittest.TestCase):
     def test0(self):
         expr0 = Group(X(), 'a', 'b')
@@ -479,10 +589,10 @@ class TestGroup(unittest.TestCase):
     def test1(self):
         expr0 = Group(X(), 'a', 'b', Include('abc'))
         expr1 = Group(expr0, 'uuu', 'uuu', Exclude(Seq('a', 'z')))
-        expr2 = Group(expr1, 'mm', Any(expr0, expr1, 'fooo'), 'uuuuu')
+        expr2 = Group(expr1, 'mm', Join(expr0, expr1, 'fooo'), 'uuuuu')
 
-        expr3 = Group(expr2, 'uu', Repeat('hehe', 2))
-        expr4 = Any(expr0, expr1, expr2, expr3)
+        expr3 = Group(expr2, 'uu')
+        expr4 = Join(expr0, expr1, expr2, expr3)
         expr5 = Join(expr4, expr0, expr1, expr2, expr3, expr4)
 
         regstr = expr5.mkregex()
@@ -493,31 +603,42 @@ class TestGroup(unittest.TestCase):
         self.assertEqual(yregex.mkregex(), regstr)
 
     def test2(self):
-        expr0 = Group(X(), 'a', 'b', Include('abc'))
-        expr1 = Group(expr0, 'uuu', 'uuu', Exclude(Seq('a', 'z')))
-        expr2 = Group(expr1, 'mm', Any(expr0, expr1, 'fooo'), 'uuuuu')
-        expr3 = Group(expr2, 'uu', Repeat('hehe', 2))
-        expr4 = Any(expr0, expr1, expr2, expr3)
-
-        expr5 = Group('fooooo', expr4, expr3, expr0, 'hheheh')
-        expr6 = Join(expr4, expr0, expr1, expr2, expr3, 
-        expr4, expr5, 'ooo', expr5)
-
-        expr7 = Join(expr0, expr1, expr5, expr4, 
-        expr5, 'ooo', expr5)
-
-        expr8 = OneOrMore(expr7)
-        expr9 = Join(expr0, expr1, expr2, expr3, expr4, 
-        expr5, expr6, expr7, expr8)
-
-        regstr = expr9.mkregex()
+        expr0 = Group('a')
+        expr1 = Group('c', Group(expr0, 'd'))
+        expr2 = Group(expr1, 'e')
+        expr3 = Join(expr0, expr1, expr2)
 
+        regstr = expr3.mkregex()
         yregex = xmake(regstr)
-        print('yregex', yregex)
-        yregex.test()
+        expr3.test()
+
         yregex.hits()
+        yregex.test()
         self.assertEqual(yregex.mkregex(), regstr)
 
+    def test3(self):
+        expr0 = Group('abc')
+        expr1 = Group(expr0, 'efg')
+        expr2 = Group(expr0, expr1)
+        regstr = expr2.mkregex()
+        yregex = xmake(regstr)
+
+        yregex.test()
+
+        # Eacc should be capable of reading back the 
+        # serialized string.
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test4(self):
+        expr0 = Group('ab')
+        expr1 = Group(expr0, expr0)
+        regstr = expr1.mkregex()
+        print(regstr)
+        yregex = xmake(regstr)
+        print('yRegex:', yregex)
+        # yregex.test()
+        # self.assertEqual(yregex.mkregex(), regstr)
+
 class TestNamedGroup(unittest.TestCase):
     def test0(self):
         expr0 = NamedGroup('beta', 'X', X(), 'B')
@@ -546,7 +667,7 @@ class TestNamedGroup(unittest.TestCase):
         # Check if it works for nested named groups.
         expr0 = NamedGroup('alpha', 'X', OneOrMore(Group('a', 'b')), 'B')
         expr1 = NamedGroup('beta', 'Lets be overmen.')
-        expr2 = NamedGroup('gamma', OneOrZero(expr1), 'rs', OneOrMore('rs'))
+        expr2 = NamedGroup('gamma', OneOrMore(expr1), 'rs', OneOrMore('rs'))
 
         expr3 = NamedGroup('delta', expr0, expr1, expr2, 'hoho')
         expr4 = Join(expr0, expr1, expr0, expr1, expr2, expr3)
@@ -585,7 +706,7 @@ class TestRepeat(unittest.TestCase):
         yregex = xmake(regstr)
 
         yregex.test()
-        # yregex.hits()
+        yregex.hits()
         self.assertEqual(yregex.mkregex(), regstr)
 
     def test1(self):
@@ -601,63 +722,125 @@ class TestRepeat(unittest.TestCase):
         yregex = xmake(regstr)
 
         yregex.test()
-        # yregex.hits()
+        yregex.hits()
         self.assertEqual(yregex.mkregex(), regstr)
 
-class TestZeroOrMore(unittest.TestCase):
-    def test0(self):
-        expr0 = NamedGroup('oooo', Repeat(Any('a', X(), 'b')))
-        expr1 = Any(expr0, 'm', 'n', Group('oooo'), Group(expr0, X(), '12oooo', X()))
-        expr2 = Repeat(expr1)
-
-        expr3 = Join(expr0, expr1, expr2)
-        expr4 = Join(expr0, X(), 'ooo', X(), expr1, expr2, expr3)
-        expr5 = Any(expr0, expr1, expr2, expr3, expr4)
-        expr6 = Any(expr0, expr1, expr2, expr3, 
-        expr4, expr5, 'hahah', 'hoohoho', X())
-
-        expr7 = ZeroOrMore(expr6)
-        expr8 = Join(expr7, expr6, expr5, expr4, 
-        expr3, expr2, expr1, expr0)
-
-        regstr = expr8.mkregex()
+    def test2(self):
+        regstr = r'a{1,2}'
         yregex = xmake(regstr)
-
         yregex.test()
         yregex.hits()
         self.assertEqual(yregex.mkregex(), regstr)
 
+    def test3(self):
+        regstr = r'\*{1,2}?c*'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test4(self):
+        regstr = r'\*{1,8}?\&+?'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test5(self):
+        regstr = r'(\*cd){1,8}?\&+?\1+?'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test6(self):
+        regstr = r'((\*cd){1,8}?\&)+?(\1?ecdeeec+)+?'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test7(self):
+        regstr = r'((\*cd{1,}){1,8}?\&)+?(\1?ec?d){4,4}?'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test8(self):
+        regstr = r'(((\*cd{1,}){1,8}?\&)+?){3,3}?(\1?ecd){4,4}?'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test9(self):
+        regstr = r'a{1,3}c{3,3}?e{0,2}?c+?e{0,4}?'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test10(self):
+        regstr = r'(a{1,3}c{3,3}?e{2,2}?c+?e{0,4}?)\1{1,2}?'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test11(self):
+        regstr = r'((a{1,3}c{3,3}?e{2,3}?c+?e{1,4}?)\2{1,2}?)'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test12(self):
+        regstr = r'a'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+class TestZeroOrMore(unittest.TestCase):
+    def test0(self):
+        expr0 = NamedGroup('alpha', Repeat(Any('a', X())))
+        expr1 = Join(expr0, 'm', Group('oooo'))
+        expr2 = Any(expr0, expr1)
+
+        expr7 = ZeroOrMore(expr2)
+        expr8 = Join(expr7, expr2)
+
+        regstr = expr8.mkregex()
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+        self.assertEqual(yregex.mkregex(), regstr)
+        
     def test1(self):
-        expr0 = NamedGroup('a999', ZeroOrMore('ooo'), Repeat(Any('a', X(), 'b')))
-        expr1 = Any(expr0, 'm', 'n', ZeroOrMore(Any('a', 'b')), 
-        Group('oooo'), Group(expr0, X(), '12oooo', X()))
+        expr0 = NamedGroup('a999', Repeat('a'))
+
+        expr1 = Join(expr0, ZeroOrMore('a'), 
+        Group('oooo'), Group(expr0))
 
         expr2 = Repeat(expr1)
-        expr3 = Join(expr0, expr1, expr2, ZeroOrMore('ooo'), X(), ZeroOrMore('heheh'))
-        expr4 = Join(expr0, X(), 'ooo', X(), expr1, expr2, expr3)
 
-        expr5 = Any(expr0, expr1, expr2, ZeroOrMore(X()), expr2, expr3, expr4)
-        expr6 = Any(expr0, expr1, expr2, expr3, 
-        expr4, expr5, 'hahah', 'hoohoho', X())
-
-        expr7 = ZeroOrMore(expr6)
-
-        expr8 = Join(expr7, expr6, expr5, 
-        expr4, expr3, expr2, expr1, expr0)
-
-        expr9 = NamedGroup('fooooooo', expr8, 'foooo')
-        expr10 = ZeroOrMore(expr9)
-
-        regstr = expr10.mkregex()
+        regstr = expr2.mkregex()
         yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+        self.assertEqual(yregex.mkregex(), regstr)
 
+    def test2(self):
+        regstr = r'\*{1,3}(((a[0-9]c)\3{1,3}))((\$[a-z]\#)\2{1,3})\*{1,3}'
+        yregex = xmake(regstr)
         yregex.test()
         yregex.hits()
         self.assertEqual(yregex.mkregex(), regstr)
 
 class TestConsumeNext(unittest.TestCase):
     def test0(self):
-        expr0 = ConsumeNext(Group(X(), OneOrZero('alpha')), 
+        expr0 = ConsumeNext(Group(X(), OneOrMore('alpha')), 
         Group(X(), 'bar', X()))
         expr1 = Any(expr0, X(), '123')
         regstr = expr1.mkregex()
@@ -665,12 +848,11 @@ class TestConsumeNext(unittest.TestCase):
 
         with self.assertRaises(re.error):
             yregex.test()
-        yregex.hits()
         self.assertEqual(yregex.mkregex(), regstr)
 
     def test1(self):
         expr0 = ConsumeNext(Group(X(), 'bar', X()), 
-        Group(X(), OneOrZero('alpha')))
+        Group(X(), OneOrMore('alpha')))
 
         expr1 = Any(expr0, X(), '123')
         expr2 = ConsumeNext(Group(X(), '579', X()), expr1)
@@ -683,14 +865,14 @@ class TestConsumeNext(unittest.TestCase):
 
     def test2(self):
         expr0 = ConsumeNext(Group(X(), 'bar', X()), 
-        Group(X(), OneOrZero('alpha')))
+        Group(X(), OneOrMore('alpha')))
         expr1 = Any(expr0, X(), '123')
         expr2 = ConsumeNext(Group(X(), '579', X()), expr1)
 
         expr3 = ConsumeBack(expr2, 
-        Group(ZeroOrMore(expr2), 'aaaaa', 'bbbb', X()))
+        Group(OneOrMore(expr2), 'aaaaa', 'bbbb', X()))
 
-        expr3 = Join(expr3, 'aaaa', X(), OneOrZero('aaaa'))
+        expr3 = Join(expr3, 'aaaa', X(), OneOrMore('aaaa'))
         regstr = expr3.mkregex()
         yregex = xmake(regstr)
 
@@ -770,8 +952,7 @@ class TestRegexComment(unittest.TestCase):
         self.assertEqual(yregex.mkregex(), regstr)
 
     def test4(self):
-        regstr = r'''[a-z]*(?#aiosdu)((ab)*)?([0-9]+)\\
-        1sdius\2(123)(?#....aaa\)[abcde])\1aa'''
+        regstr = r'[a-z](?#aiosdu)([0-9]+)\1sdius\1(123)(?#....aaa\)[abcde])\1aa'
 
         yregex = xmake(regstr)
 
@@ -779,7 +960,7 @@ class TestRegexComment(unittest.TestCase):
         yregex.hits()
         self.assertEqual(yregex.mkregex(), regstr)
 
-    def test4(self):
+    def test5(self):
         regstr = 'a(?#aiosdu)*b'
         yregex = xmake(regstr)
 
@@ -787,5 +968,120 @@ class TestRegexComment(unittest.TestCase):
         yregex.hits()
         self.assertEqual(yregex.mkregex(), regstr)
 
+class TestNonCapture(unittest.TestCase):
+    def test0(self):
+        regstr = r'(?:ee)'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test1(self):
+        regstr = '((?:.+)cc(ee.*))+'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test2(self):
+        regstr = '((?:.+)cc(ee.*))+\1+c\2+e'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test2(self):
+        regstr = '((?:fooobar.+)cc(ee.*))+\1+c\2+e'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+class TestWord(unittest.TestCase):
+    def test0(self):
+        regstr = r'ee\*@\w+hehe'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test1(self):
+        regstr = r'(e\&e\*@\w*cc)+(?:cee\w+)uu?\1'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+class TestNotWord(unittest.TestCase):
+    def test0(self):
+        regstr = r'foo\W+bar'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test1(self):
+        regstr = r'(\*\W+bar)+(@\1+)\&\2{1,3}'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+class TestCaret(unittest.TestCase):
+    def test0(self):
+        regstr = r'^eudof.c+'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test1(self):
+        regstr = r'^(\*\^ee)+'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test2(self):
+        regstr = r'^(\*\^ee)+c*\..'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+
+        self.assertEqual(yregex.mkregex(), regstr)
+
+class TestDollar(unittest.TestCase):
+    def test0(self):
+        regstr = r'^eudof.c+$'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test1(self):
+        regstr = r'^(\*\^ee)+$'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test2(self):
+        regstr = r'^(\*\^ee)+c*\..$'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+
+        self.assertEqual(yregex.mkregex(), regstr)
+
+    def test2(self):
+        regstr = r'((\*\^ee)+c*\..)*$'
+        yregex = xmake(regstr)
+        yregex.test()
+        yregex.hits()
+
+        self.assertEqual(yregex.mkregex(), regstr)
+
 if __name__ == '__main__':
     unittest.main()
\ No newline at end of file
